--------------------------------------------------------------------------
--                                                                      --
--                                                                      --
--               miniMIPS Processor : Register bank                     --
--                                                                      --
--                                                                      --
--                                                                      --
-- Authors : Hangouet  Samuel                                           --
--           Jan       Sbastien                                        --
--           Mouton    Louis-Marie                                      --
--           Schneider Olivier                                          --
--                                                                      --
--                                                          june 2003   --
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

library work;
use work.pack_mips.all;

entity banc is
port (
       clock : in std_logic;
       reset : in std_logic;

       -- Register addresses to read
       reg_src1 : in bus5;
       reg_src2 : in bus5;

       -- Register address to write and its data
       reg_dest : in bus5;
       donnee   : in bus32;

       -- Write signal
       cmd_ecr  : in std_logic;

       -- Bank outputs
       data_src1 : out bus32;
       data_src2 : out bus32;

       stop_all  : out std_logic
     );
end banc;


architecture rtl of banc is

    -- The register bank
    type tab_reg is array (1 to 31) of bus32;
    signal registres : tab_reg;

    signal adr_src1 : integer range 0 to 31;
    signal adr_src2 : integer range 0 to 31;
    signal adr_dest : integer range 0 to 31;

    signal wait_reg : bus32;  -- register 31 is wait reg
    signal wr_val   : bus32;  -- save value waiting to be written to WR
    signal wr_ecr, wr_dec : std_logic;
    signal wr_src : std_logic;

    signal cs : std_logic;    
begin

    adr_src1 <= to_integer(unsigned(reg_src1));
    adr_src2 <= to_integer(unsigned(reg_src2));
    adr_dest <= to_integer(unsigned(reg_dest));


    data_src1 <= (others => '0') when adr_src1=0 else
                wait_reg  when adr_src1=31 else
                 registres(adr_src1);

    data_src2 <= (others => '0') when adr_src2=0 else
                 wait_reg when adr_src2=31 else
                 registres(adr_src2);

    -- handle regular register writing
    process(clock)
    begin
        if clock = '1' and clock'event then
            if reset='1' then
                for i in 1 to 30 loop
                    registres(i) <= (others => '0');
                end loop;
            else
                if cmd_ecr = '1' and adr_dest /= 0 and adr_dest /= 31 then	--write req
                    registres(adr_dest) <= donnee;
                end if;
            end if;
        end if;
    end process;



    -- handle stop_all condition and setup new value to write
    process(reset,wr_ecr, wr_dec)
    begin
            if reset = '1' then
                cs <= '0';
                stop_all <= '0';
                wr_val  <= X"00000000";
            else
                case cs is
                    when '0' =>
                        wr_src <= '0';  -- store input
                        if wr_ecr = '1' and wr_dec = '1' then
                            wr_val <= donnee;
                            stop_all <= '1';
                            cs <= '1';
                        end if;
                    when '1' =>
                        wr_src <= '1';  -- store register
                        if wr_dec = '0' then
                            stop_all <= '0';
                            cs <= '0';
                        end if;
                    when others => 
                        stop_all <= '0';
                        cs <= '0';
                end case;
            end if;
    end process;
                        --          -- means there was a previous write req
    wr_ecr <= '1' when (cmd_ecr = '1' and adr_dest = 31) or  cs = '1' else
              '0';

    wr_dec <= '0' when wait_reg = X"000000" else
              '1';

    -- wait register controller
    process(clock,reset,wr_ecr, wr_dec)
    begin
        if clock = '1' and clock'event then
            if reset = '1' then
                wait_reg <= (others =>'0');
            elsif wr_ecr = '1' and wr_dec = '0' then
                if wr_src = '0' then
                    wait_reg <= donnee;
                else
                    wait_reg <= wr_val;
                end if;
            elsif wr_dec = '1' then
                wait_reg <= wait_reg - 1;
            end if;
        end if;
    end process;

end rtl;
