--------------------------------------------------------------------------
--                                                                      --
--                                                                      --
--     Processor miniMIPS : Enumerations and components declarations    --
--                                                                      --
--                                                                      --
--                                                                      --
-- Authors : Hangouet  Samuel                                           --
--           Jan       Sbastien                                        --
--           Mouton    Louis-Marie                                      --
--           Schneider Olivier                                          --
--                                                                      --
--                                                          june 2003   --
--------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;

package pack_mips is

    -- Type signal on n bits
    subtype bus64 is std_logic_vector(63 downto 0);
    subtype bus33 is std_logic_vector(32 downto 0);
    subtype bus32 is std_logic_vector(31 downto 0);
    subtype bus31 is std_logic_vector(30 downto 0);
    subtype bus26 is std_logic_vector(25 downto 0);
    subtype bus24 is std_logic_vector(23 downto 0);
    subtype bus16 is std_logic_vector(15 downto 0);
    subtype bus8 is std_logic_vector(7 downto 0);
    subtype bus6 is std_logic_vector(5 downto 0);
    subtype bus5 is std_logic_vector(4 downto 0);
    subtype bus4 is std_logic_vector(3 downto 0);
    subtype bus2 is std_logic_vector(1 downto 0);
--    subtype bus1 is std_logic;  -- causes problems with synthesis and simulations

    -- Address of a register type
    subtype adr_reg_type is std_logic_vector(5 downto 0);

    -- Coding of the level of data availability for UR
    subtype level_type is std_logic_vector(1 downto 0);
    constant LVL_DI  : level_type := "11";  -- Data available from the op2 of DI stage
    constant LVL_EX  : level_type := "10";  -- Data available from the data_ual register of EX stage
    constant LVL_MEM : level_type := "01";  -- Data available from the data_ecr register of MEM stage
    constant LVL_REG : level_type := "00";  -- Data available only in the register bank

    -- Different values of cause exceptions
    constant IT_NOEXC : bus32 := X"00000000";
    constant IT_ITMAT : bus32 := X"00000001";
    constant IT_OVERF : bus32 := X"00000002";
    constant IT_ERINS : bus32 := X"00000004";
    constant IT_BREAK : bus32 := X"00000008";
    constant IT_SCALL : bus32 := X"00000010";


    -- Operation type of the coprocessor system (only the low 16 bits are valid)
    constant SYS_NOP    : bus32 := X"0000_0000";
    constant SYS_MASK   : bus32 := X"0000_0001";
    constant SYS_UNMASK : bus32 := X"0000_0002";
    constant SYS_ITRET  : bus32 := X"0000_0004";

    -- Type for the alu control
    subtype alu_ctrl_type is std_logic_vector(27 downto 0);


-- lower bits = rightmost 16 bits?
-- upper bits = leftmost 16 bits?

    -- Arithmetical operations
    constant OP_ADD   : alu_ctrl_type := "1000000000000000000000000000"; -- op1 + op2 signed
    constant OP_ADDU  : alu_ctrl_type := "0100000000000000000000000000"; -- op1 + op2 unsigned
    constant OP_SUB   : alu_ctrl_type := "0010000000000000000000000000"; -- op1 - op2 signed
    constant OP_SUBU  : alu_ctrl_type := "0001000000000000000000000000"; -- op1 - op2 unsigned
    -- Logical operations
    constant OP_AND   : alu_ctrl_type := "0000100000000000000000000000"; -- logical AND
    constant OP_OR    : alu_ctrl_type := "0000010000000000000000000000"; -- logical OR
    constant OP_XOR   : alu_ctrl_type := "0000001000000000000000000000"; -- logical XOR
    constant OP_NOR   : alu_ctrl_type := "0000000100000000000000000000"; -- logical NOR
    -- Tests : result to one if ok
    constant OP_SLT   : alu_ctrl_type := "0000000010000000000000000000"; -- op1 < op2 (signed)
    constant OP_SLTU  : alu_ctrl_type := "0000000001000000000000000000"; -- op1 < op2 (unsigned)
    constant OP_EQU   : alu_ctrl_type := "0000000000100000000000000000"; -- op1 = op2
    constant OP_NEQU  : alu_ctrl_type := "0000000000010000000000000000"; -- op1 /= op2
    constant OP_SNEG  : alu_ctrl_type := "0000000000001000000000000000"; -- op1 < 0
    constant OP_SPOS  : alu_ctrl_type := "0000000000000100000000000000"; -- op1 > 0
    constant OP_LNEG  : alu_ctrl_type := "0000000000000010000000000000"; -- op1 <= 0
    constant OP_LPOS  : alu_ctrl_type := "0000000000000001000000000000"; -- op1 >= 0
    -- Multiplications
    constant OP_MULT  : alu_ctrl_type := "0000000000000000100000000000"; -- op1 * op2 signed (load lower bits)
    constant OP_MULTU : alu_ctrl_type := "0000000000000000010000000000"; -- op1 * op2 unsigned (load lower bits)
    -- Shifts
    constant OP_SLL   : alu_ctrl_type := "0000000000000000001000000000"; -- logical shift left
    constant OP_SRL   : alu_ctrl_type := "0000000000000000000100000000"; -- logical shift right
    constant OP_SRA   : alu_ctrl_type := "0000000000000000000010000000"; -- arithmetic shift right
    constant OP_LUI   : alu_ctrl_type := "0000000000000000000001000000"; -- load immediate value
    -- Access to internal registers
    constant OP_MFHI  : alu_ctrl_type := "0000000000000000000000100000"; -- read upper bits
    constant OP_MFLO  : alu_ctrl_type := "0000000000000000000000010000"; -- read lower bits
    constant OP_MTHI  : alu_ctrl_type := "0000000000000000000000001000"; -- write upper bits
    constant OP_MTLO  : alu_ctrl_type := "0000000000000000000000000100"; -- write lower bits
    -- Operations which do nothing but are useful
    constant OP_OUI   : alu_ctrl_type := "0000000000000000000000000010"; -- puts a 1 bit on exit (lower bits)
    constant OP_OP2   : alu_ctrl_type := "0000000000000000000000000001"; -- copy operand 2 on exit



    -- Starting boot address (after reset)
    constant ADR_INIT : bus32 := X"00000000";
    constant INS_NOP : bus32 := X"00000000";


    -- Internal component of the pipeline stage

    component alu
    port (
        clock : in std_logic;
        reset : in std_logic;
        op1 : in bus32;
        op2 : in bus32;
        ctrl : in alu_ctrl_type;

        res : out bus32;
        overflow : out std_logic
    );
    end component;

    -- Pipeline stage components

    component pps_pf
    port (
        clock : in std_logic;
        reset : in std_logic;
        stop_all : in std_logic;

        bra_cmd  : in std_logic;
        bra_adr  : in bus32;
        exch_cmd : in std_logic;
        exch_adr : in bus32;

        stop_pf : in std_logic;

        PF_pc : out bus32
    );
    end component;


    component pps_ei
    port (
        clock : in std_logic;
        reset : in std_logic;
        clear  : in std_logic;
        stop_all : in std_logic;

        stop_ei : in std_logic;
        genop : in std_logic;

        CTE_instr : in bus32;
        ETC_adr : out bus32;

        PF_pc : in bus32;

        EI_instr : out bus32;
        EI_adr : out bus32;
        EI_it_ok : out std_logic
    );
    end component;


    component pps_di
    port (
        clock : in std_logic;
        reset : in std_logic;
        stop_all : in std_logic;
        clear : in std_logic;

        bra_detect : out std_logic;

        adr_reg1 : out adr_reg_type;
        adr_reg2 : out adr_reg_type;
        use1 : out std_logic;
        use2 : out std_logic;

        stop_di : in std_logic;
        data1 : in bus32;
        data2 : in bus32;

        EI_adr : in bus32;
        EI_instr : in bus32;
        EI_it_ok : in std_logic;

        DI_bra : out std_logic;
        DI_link : out std_logic;
        DI_op1 : out bus32;
        DI_op2 : out bus32;
        DI_code_ual : out alu_ctrl_type;
        DI_offset : out bus32;
        DI_adr_reg_dest : out adr_reg_type;
        DI_ecr_reg : out std_logic;
        DI_mode : out std_logic;
        DI_op_mem : out std_logic;
        DI_r_w : out std_logic;
        DI_adr : out bus32;
        DI_exc_cause : out bus32;
        DI_level : out level_type;
        DI_it_ok : out std_logic
    );
    end component;


    component pps_ex
    port (
        clock : in std_logic;
        reset : in std_logic;
        stop_all : in std_logic;
        clear : in std_logic;

        DI_bra : in std_logic;
        DI_link : in std_logic;
        DI_op1 : in bus32;
        DI_op2 : in bus32;
        DI_code_ual : in alu_ctrl_type;
        DI_offset : in bus32;
        DI_adr_reg_dest : in adr_reg_type;
        DI_ecr_reg : in std_logic;
        DI_mode : in std_logic;
        DI_op_mem : in std_logic;
        DI_r_w : in std_logic;
        DI_adr : in bus32;
        DI_exc_cause : in bus32;
        DI_level : in level_type;
        DI_it_ok : in std_logic;

        EX_adr : out bus32;
        EX_bra_confirm : out std_logic;
        EX_data_ual : out bus32;
        EX_adresse : out bus32;
        EX_adr_reg_dest : out adr_reg_type;
        EX_ecr_reg : out std_logic;
        EX_op_mem : out std_logic;
        EX_r_w : out std_logic;
        EX_exc_cause : out bus32;
        EX_level : out level_type;
        EX_it_ok : out std_logic
    );
    end component;


    component pps_mem
    port (
        clock : in std_logic;
        reset : in std_logic;
        stop_all : in std_logic;
        clear : in std_logic;

        MTC_data : out bus32;
        MTC_adr : out bus32;
        MTC_r_w : out std_logic;
        MTC_req : out std_logic;
        CTM_data : in bus32;

        EX_adr : in bus32;
        EX_data_ual : in bus32;
        EX_adresse : in bus32;
        EX_adr_reg_dest : in adr_reg_type;
        EX_ecr_reg : in std_logic;
        EX_op_mem : in std_logic;
        EX_r_w : in std_logic;
        EX_exc_cause : in bus32;
        EX_level : in level_type;
        EX_it_ok : in std_logic;

        MEM_adr : out bus32;
        MEM_adr_reg_dest : out adr_reg_type;
        MEM_ecr_reg : out std_logic;
        MEM_data_ecr : out bus32;
        MEM_exc_cause : out bus32;
        MEM_level : out level_type;
        MEM_it_ok : out std_logic
    );
    end component;


    component renvoi  -- reference
    port (
        adr1 : in adr_reg_type;
        adr2 : in adr_reg_type;
        use1 : in std_logic;
        use2 : in std_logic;

        data1 : out bus32;
        data2 : out bus32;
        alea : out std_logic;

        DI_level : in level_type;
        DI_adr : in adr_reg_type;
        DI_ecr : in std_logic;
        DI_data : in bus32;

        EX_level : in level_type;
        EX_adr : in adr_reg_type;
        EX_ecr : in std_logic;
        EX_data : in bus32;

        MEM_level : in level_type;
        MEM_adr : in adr_reg_type;
        MEM_ecr : in std_logic;
        MEM_data : in bus32;

        interrupt : in std_logic;

        write_data : out bus32;
        write_adr : out bus5;
        write_GPR : out std_logic;
        write_SCP : out std_logic;

        read_adr1 : out bus5;
        read_adr2 : out bus5;
        read_data1_GPR : in bus32;
        read_data1_SCP : in bus32;
        read_data2_GPR : in bus32;
        read_data2_SCP : in bus32
    );
    end component;


    component banc  -- register file
    port (
        clock : in std_logic;
        reset : std_logic;

        reg_src1 : in bus5;
        reg_src2 : in bus5;

        reg_dest : in bus5;
        donnee   : in bus32; -- data in

        cmd_ecr  : in std_logic;  -- write enable

        data_src1 : out bus32;	-- data out 1
        data_src2 : out bus32;    -- data out 2

	stop_all   : out std_logic
    );
    end component;


    component bus_ctrl
    port
    (
        clock : std_logic;
        reset : std_logic;

        -- Interruption in the pipeline
        interrupt      : in std_logic;

        -- Interface for the Instruction Extraction Stage
        adr_from_ei    : in bus32;      -- The address of the data to read
        instr_to_ei    : out bus32;     -- Instruction from the memory

        -- Interface with the MEMory Stage
        req_from_mem   : in std_logic;  -- Request to access the ram
        r_w_from_mem   : in std_logic;  -- Read/Write request
        adr_from_mem   : in bus32;      -- Address in ram
        data_from_mem  : in bus32;      -- Data to write in ram
        data_to_mem    : out bus32;     -- Data from the ram to the MEMory stage

        load_to_video  : out std_logic;
        vsync_to_video : out std_logic;
        hsync_to_video : out std_logic;
        blank_to_video : out std_logic;
        byte_to_video  : out bus8;

        -- Pipeline progress control signal
        stop_all       : in std_logic
    );
    end component;


    component syscop
    port
    (
        clock         : in std_logic;
        reset         : in std_logic;

        MEM_adr       : in bus32;
        MEM_exc_cause : in bus32;
        MEM_it_ok     : in std_logic;

        it_mat        : in std_logic;

        interrupt     : out std_logic;
        vecteur_it    : out bus32;

        write_data    : in bus32;
        write_adr     : in bus5;
        write_SCP     : in std_logic;

        read_adr1     : in bus5;
        read_adr2     : in bus5;
        read_data1    : out bus32;
        read_data2    : out bus32
    );
    end component;


    component minimips
    port (
        clock    : in std_logic;
        reset    : in std_logic;

        load_to_video  : out std_logic;
        vsync_to_video : out std_logic;
        hsync_to_video : out std_logic;
        blank_to_video : out std_logic;
        byte_to_video  : out bus8;

        it_mat   : in std_logic
    );
    end component;


    component video_controller
        port (
        pixel_clock     : in std_logic;
	reset           : in std_logic;

        --minimips output : input video controller
	RT_Font_Byte    : in std_logic_vector(7 downto 0);
	RT_Shift_Load   : in std_logic;
	RT_Blank        : in std_logic;
	RT_Hsync        : in std_logic;
	RT_VSync        : in std_logic;

        --video connections
	VIDOUT_CLK     : out std_logic;
	VIDOUT_RED     : out std_logic_vector (9 downto 0);
	VIDOUT_GREEN   : out std_logic_vector (9 downto 0);
	VIDOUT_BLUE    : out std_logic_vector (9 downto 0);
	VIDOUT_BLANK_N : out std_logic;
	VIDOUT_HSYNC_N : out std_logic;
	VIDOUT_VSYNC_N : out std_logic
    );
    end component;

    component rtproc
    port (

	-- signals to video DAC
	pixel_clock      : in std_logic;
	VIDOUT_CLK     : out std_logic;
	VIDOUT_RCR     : out std_logic_vector (9 downto 0);
	VIDOUT_GY      : out std_logic_vector (9 downto 0);
	VIDOUT_BCB     : out std_logic_vector (9 downto 0);
	VIDOUT_BLANK_N : out std_logic;
	VIDOUT_HSYNC_N : out std_logic;
	VIDOUT_VSYNC_N : out std_logic;

	BAR_LED        : out std_logic_vector (9 downto 0)
    );
    end component;

end pack_mips;
